/*
   program : recon,SheppLogan

   $Source: /home/yee/C/RCS/ImageHeader.h,v $
   $Author: yee $
   $Date: 1999/02/10 16:39:47 $
   $Revision: 3.10 $

   This file contains the format of the reconstruction header.  In
   order to minimize the need for knowledge about the specific format
   of the header, as much of the code using the header information is
   #defined in macros below.

   Parameters : none

   Assumptions : none known

   Bugs : none known

   To Do :
   1) support float vs. int images.

   $Log: ImageHeader.h,v $
   Revision 3.10  1999/02/10 16:39:47  yee
   *** empty log message ***

   Revision 3.9  1999/01/16 16:33:16  yee
   *** empty log message ***

   Revision 3.6  1998/08/22 15:37:39  yee
   all geometries work!

   char ImageHeader[] = "$Id: ImageHeader.h,v 3.10 1999/02/10 16:39:47 yee Exp $";

   Copyright 1997,1998,1999 by Conway Yee.  All rights reserved.
*/
/*****************************************************************************/
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "style.h"
#include "byte.h"
#include "mutil.h"

#ifndef IMAGE_HEADER_H
#define IMAGE_HEADER_H

#define IMAGE_MAGIC 0x4321

#define NDIM 2

#if defined(DOUBLE)
#define IMAGE_FLOAT double
#define IMAGE_INT   long
#define IMAGE_COMPLEX lcomplex
#else SINGLE
#define IMAGE_FLOAT float
#define IMAGE_INT   int
#define IMAGE_COMPLEX complex
#endif /* single vs. double precision */

  typedef struct HeaderString {
    char *s;           /* data string */
    unsigned int size; /* size of string excluding final NULL */
  } HeaderString;

  typedef struct ImageHeader {
    unsigned int hsize;   /* header size in bytes */
    unsigned int fsize;   /* sizeof(float) */
    CHOICE type;          /* image type */
    double D;             /* dist between focal point and center of rotation */
    unsigned int ndim;    /* dimensionality of image (e.g. 2-D image) */
    int *lim;             /* limits in each dimension */
    int *dim;             /* number of elements in each dimension */
    double *tau;          /* distance between samples in image */
    HeaderString name;    /* patient name or filename */
    HeaderString mr;      /* patient medical record number */
    HeaderString loc;     /* patient location */
    FILE *fptr;           /* pointer to input file containing data */
    union {               /* pointer to data array */
      IMAGE_FLOAT   **f;
      IMAGE_INT     **i;
      IMAGE_COMPLEX **c;
    } data;
  } ImageHeader;
                              /* low and high projection number
                                 low and high element in each projection

                                 Since the elements of each projection must
                                 be in a contiguous of memory, PNUM must come
                                 before ENUM since mdmalloc() and mdio()
                                 assumes that the last index is contiguous. */
  enum{LO_PROJ,HI_PROJ,LO_ELE,HI_ELE};
  enum{PROJ,ELE};

  enum{LOX,HIX,LOY,HIY};  /* for images, hi and low pixels in X,Y directions */
  enum{X,Y,Z};            /* X,Y directions */

#define ReallocImageHeader(h,newdim) {                               \
            if(h->ndim < newdim) {                                     \
              h->ndim=newdim;                                          \
              h->tau=(double *)realloc(h->tau,newdim*sizeof(double));  \
              h->lim=(int *)realloc(h->lim,newdim*2*sizeof(int));      \
              h->dim=(int *)realloc(h->dim,newdim*sizeof(int));        \
              memset(h->tau,(char)NULL,newdim*sizeof(double));         \
              memset(h->lim,(char)NULL,newdim*2*sizeof(int));          \
            }                                                          \
          }

#define ImageHeaderEditPersonalData(field,string) { \
            free(field.s);                            \
            field.size=strlen(string);                \
            field.s=calloc(1,field.size+1);           \
            strcpy(field.s,string);                   \
          }

#define ImageHeaderClonePersonalData(inheader,outheader) {              \
            ImageHeaderEditPersonalData(outheader->name,inheader->name.s) \
            ImageHeaderEditPersonalData(outheader->loc,inheader->loc.s)   \
            ImageHeaderEditPersonalData(outheader->mr,inheader->mr.s)     \
          }                                                   

#define ImageHeaderPrintPersonalData(fptr,h) {                      \
    fprintf(fptr,"recon2img : patient name       : %s\n",h->name.s);  \
    fprintf(fptr,"recon2img : patient location   : %s\n",h->loc.s);   \
    fprintf(fptr,"recon2img : patient identifier : %s\n",h->mr.s);    \
  }

#define AllocateImageHeader(h,newdim) {                     \
            h=(ImageHeader *)calloc(1,sizeof(ImageHeader));   \
            h->mr.s   = (char *)calloc(1,1);                  \
            h->loc.s  = (char *)calloc(1,1);                  \
            h->name.s = (char *)calloc(1,1);                  \
            ReallocImageHeader(h,newdim);                     \
            h->fsize = sizeof(IMAGE_FLOAT);                   \
          }

#define FreeImageHeader(h) { \
            free(h->name.s);   \
            free(h->mr.s);     \
            free(h->loc.s);    \
            free(h->lim);      \
            free(h->dim);      \
            free(h->tau);      \
            free(h);           \
          }
                                /* CT type codes 
                                       - PARALLEL beam
                                       - equiANGLE fan beam
                                       - COLINear fan beam */
  enum{IMAGE_TYPE_NOT_SET,PARALLEL,ANGLE,COLIN,IMAGE,MAX_IMAGE_TYPE};

  EXTERN PUBLIC CODE ioImageHeader(ImageHeader *,size_t (*)(),FILE *);

#endif IMAGE_HEADER_H
